# -*- coding: utf-8 -*-

# XXX from zzn_extremes import Min, Any, Max
import unittest2
from unittest2 import FunctionTestCase, SkipTest, skip, skipIf
from unittest2 import skipUnless, expectedFailure

VERSION = '0.1.1'

__unittest=1

__test__ = False



class Expando(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def __eq__(self, other):
        if self.__class__ != other.__class__:
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other

    def copy(self):
        return self.__class__(**self.__dict__)

    def __repr__(self):
        return 'Expando(%s)' % ','.join(
            '%s=%r' % (a,v) for a,v in self.__dict__.iteritems()
        )


class TestResourceManager(object):

    resources = []

    def managerInstance(self, name):
        for n, manager in self.resources:
            if n == name:
                return manager
        raise KeyError(name)

    def setUpClass(self):
        for name, manager in self.resources:
            res = manager.setUpClass()

    def setUp(self):
        for name, manager in self.resources:
            res = manager.setUp()
            setattr(self, name, res)
        return self

    def tearDownClass(self):
        for name, manager in self.resources:
            manager.tearDownClass()

    def tearDown(self, res):
        for name, manager in self.resources:
            res = getattr(self, name, None)
            manager.tearDown(res)


class GenericResource(TestResourceManager):

    def __init__(self, setup, teardown=None):
        self.setup = setup
        self.teardown = teardown

    def setUp(self):
        super(GenericResource, self).setUp()
        return self.setup()

    def tearDown(self, res):
        super(GenericResource, self).tearDown(res)
        if self.teardown:
            self.teardown(res)


class TestCase(unittest2.TestCase):

    resources = []

    @classmethod
    def setUpClass(cls):
        for name, manager in cls.resources:
            res = manager.setUpClass()

    def managerInstance(self, name):
        for n, manager in self.resources:
            if n == name:
                return manager
        raise KeyError(name)

    def setUp(self):
        self.setUpResources()

    def setUpResources(self):
        for name, manager in self.resources:
            self.setUpResource(name, manager)

    def setUpResource(self, name, manager):
        res = manager.setUp()
        setattr(self, name, res)

    @classmethod
    def tearDownClass(cls):
        for name, manager in cls.resources:
            manager.tearDownClass()

    def tearDown(self):
        self.tearDownResources()

    def tearDownResources(self):
        for name, manager in self.resources:
            self.tearDownResource(name, manager)

    def tearDownResource(self, name, manager):
        res = getattr(self, name, None)
        manager.tearDown(res)


    def _parseXML(self, data, type='xml'):
        from lxml import etree, html
        if isinstance(data, basestring):
            if type == 'html':
                doc = etree.fromstring(unicode(data).strip())
            else:
                doc = html.fromstring(unicode(data).strip())
        else:
            doc = data
        if etree.iselement(doc):
            return doc
        else:
            return doc.getroot()

    def assertEqualXML(self, first, second, msg=None):
        from lxml import etree
        top1 = self._parseXML(first)
        top2 = self._parseXML(second)
        root1 = top1.getroottree()
        root2 = top2.getroottree()
        diff = []
        def txteq(t1, t2):
            t1 = t1 or ''
            t2 = t2 or ''
            t1 = ' '.join(t1.strip().split())
            t2 = ' '.join(t2.strip().split())
            return t1 == t2
        def cmp(e1, e2):
            if e1 is None:
                diff.append('None x %s' % (root2.getpath(e2)))
            elif e2 is None:
                diff.append('%s x None' % (root1.getpath(e1)))
            elif e1.tag != e2.tag:
                diff.append('%s x %s' % (root1.getpath(e1), root2.getpath(e2)))
            elif e1.attrib != e2.attrib:
                diff.append('%s.attrib %s x %s' % (
                    root1.getpath(e1), e1.attrib, e2.attrib))
            elif not txteq(e1.text, e2.text):
                diff.append('%s.text %r x %r' % (
                    root1.getpath(e1), e1.text, e2.text))
            elif not txteq(e1.tail, e2.tail):
                diff.append('%s.tail %r x %r' % (
                    root1.getpath(e1), e1.tail, e2.tail))
            else:
                for c1, c2 in map(None, e1, e2):
                    cmp(c1, c2)
        cmp(top1, top2)
        if diff:
            diff.append('------------------- first ----------------------')
            diff.append(etree.tostring(top1, pretty_print=True,  method="xml"))
            diff.append('------------------- second ----------------------')
            diff.append(etree.tostring(top2, pretty_print=True,  method="xml"))
            raise self.failureException('\n'.join(diff))

    def assertEqualHTML(self, first, second, msg=None):
        top1 = self._parseXML(first, 'html')
        top2 = self._parseXML(second, 'html')
        return self.assertEqualXML(top1, top2)




class FixtureResource(TestResourceManager):

    def __init__(self, fixture):
        self.fixture = fixture

    def setUp(self):
        super(FixtureResource, self).setUp()
        self.fixture.setUp()
        return self.fixture

    def tearDown(self, res):
        super(FixtureResource, self).tearDown(res)
        res.cleanUp()

def EnvironmentVariable(varname, newvalue=None):
    from fixtures import EnvironmentVariable
    return FixtureResource(EnvironmentVariable(varname, newvalue))

def FakeLogger(name="", level=None, format=None, nuke_handlers=True):
    from logging import INFO
    from fixtures import FakeLogger
    if level is None:
        level = INFO
    return FixtureResource(FakeLogger(name, level, format, nuke_handlers))

def TempDir(rootdir=None):
    from fixtures import TempDir
    return FixtureResource(TempDir(rootdir))

def MonkeyPatch(name, new_value=None):
    from fixtures import MonkeyPatch
    return FixtureResource(MonkeyPatch(name, new_value))

def PackagePathEntry(packagename, directory):
    from fixtures import PackagePathEntry
    return FixtureResource(PackagePathEntry(packagename, directory))

def PythonPackage(packagename, modulelist, init=True):
    from fixtures import PythonPackage
    return FixtureResource(PythonPackage(packagename, modulelist, init))

def PythonPathEntry(directory):
    from fixtures import PythonPathEntry
    return FixtureResource(PythonPathEntry(directory))

def Timeout(timeout_secs, gentle):
    from fixtures import Timeout
    return FixtureResource(Timeout(timeout_secs, gentle))

