# -*- coding: utf-8 -*-

import os
import smtpd
from zzn_test import TestResourceManager


class SMTPCollector(smtpd.SMTPServer):
    """A Fake smtp server"""

    def __init__(self, localaddr, queue):
        smtpd.SMTPServer.__init__(self, localaddr, None)
        self.queue = queue

    def process_message(self, peer, mailfrom, rcpttos, data):
        self.queue.append(dict(
            peer=peer, mailfrom=mailfrom, rcpttos=rcpttos, data=data
        ))


class MailserverResource(TestResourceManager):

    def __init__(self, port=2525):
        self.port = port

    def setUp(self):
        from asyncore import loop
        from multiprocessing import Process, Manager

        super(MailserverResource, self).setUp()
        self.manager = Manager()
        self.queue = self.manager.list()
        cond = self.manager.Condition()

        def spawn(port, queue, cond):
            cond.acquire()
            col = SMTPCollector(('localhost', port), queue)
            cond.notify()
            cond.release()
            loop()

        self.process = Process(target=spawn, args=(self.port, self.queue, cond))
        cond.acquire()
        self.process.start()
        cond.wait()
        cond.release()
        return self

    def messages(self):
        return list(self.queue)

    def tearDown(self, res):
        super(MailserverResource, self).tearDown(res)
        self.process.terminate()
        self.manager.shutdown()
        del self.manager
        del self.process
